#
# == Synopsis
# A small Ruby class that decodes Ethernet packets.  See the Pcap module 
# from RAA.  A collection of EtherType constants and descriptions is 
# also provided.
#
# == Version
# 1.04, October 19th, 2006.
#
# == Example Usage 
# 
#    require 'pcap'
#    require 'Ethernet'
#
#    dev = Pcap.lookupdev
#    capture = Pcap::Capture.open_live( dev )
#    capture.loop do |packet|
#        eth = Ethernet.new( packet.raw_data )
#        puts "Dest: #{eth.dest} - Src: #{eth.src}"
#    end # of do.
#    capture.close
#             	
# == Author
# Paul Barry, The Institute of Technology, Carlow, Ireland.
# 
# == Copyright
# Copyright (c) 2006, Paul Barry.
# Licensed under GPL v2 .
# 

class Ethernet
    #
    # Given a string (as captured by the Pcap module), decode it as
    # an Ethernet packet/frame, as follows:
    #
    #   dest        - the Ethernet destination address (48 bits) in HEX.
    #   src         - the Ethernet source address (48 bits) in HEX.
    #   type_len    - the Ethernet type or length (16 bits) as net int.
    #   payload     - the Ethernet data (from 0-1500 8-bit bytes).
    # 
   
    # Constants for some common :type_len values (when Ethernet packets).
    # See http://www.iana.org/assignments/ethernet-numbers for more details.

    IPV4        = 2048          # 0x0800
    ARP         = 2054          # 0x0806
    BANYAN      = 2989          # 0x0BAD
    DECMOP      = 24578         # 0x6002
    RARP        = 32821         # 0x8035
    NETWARE     = 32823         # 0x8037
    ETHERTALK   = 32923         # 0x809B
    APPLEARP    = 33011         # 0x80F3
    TAGGED      = 33024         # 0x8100
    IPX         = 33079         # 0x8137
    NOVELL      = 33080         # 0x8138
    IPV6        = 34525         # 0x86DD
    MPLSU       = 34887         # 0x8847
    MPLSM       = 34888         # 0x8848
    PPPDICS     = 34915         # 0x8863
    PPPSESS     = 34916         # 0x8864
    ATAE        = 34978         # 0x88A2
    EAPLAN      = 34958         # 0x888E

    ETHERTYPES = {
        IPV4        => 'Internet Protocol version 4',
        ARP         => 'Address Resolution Protocol',
        BANYAN      => 'Banyan Systems',
        DECMOP      => 'DEC Maintenance Operation Protocol (MOP) Remote Console',
        RARP        => 'Reverse Address Resolution Protocol',
        NETWARE     => 'Novell Netware',
        ETHERTALK   => 'Apple EtherTalk',
        APPLEARP    => 'Apple Address Resolution Protocol',
        TAGGED      => 'IEEE 802.1G-tagged frame',
        IPX         => 'Novell IPX',
        NOVELL      => 'Novell',
        IPV6        => 'Internet Protocol version 6',
        MPLSU       => 'MPLS Unicast', 
        MPLSM       => 'MPLS Multicase',
        PPPDICS     => 'PPPoE Discovery Stage',
        PPPSESS     => 'PPPoE Session Stage',
        ATAE        => 'ATA over Ethernet',
        EAPLAN      => 'EAP over LAN (IEEE 802.1X)',
    }      

    attr_reader :dest, :src, :type_len, :payload
   
    def initialize ( frame )
        #
        # This code uses ideas (and unpack/sprintf specs) from Tim
        # Potter's NetPacket::Ethernet module (from Perl's CPAN).
        #   

        d_hi, d_lo, s_hi, s_lo, @type_len, @payload = frame.unpack( 'NnNnna*' )

        @dest = hex_format( sprintf( "%08x%04x", d_hi, d_lo ).upcase )
        @src  = hex_format( sprintf( "%08x%04x", s_hi, s_lo ).upcase )

    end # of initialize.

    private

        def hex_format( str )
            #
            # Convert 12 chars to a "readable" HEX format (as used
            # by the GNU/Linux 'ifconfig' command).
            # 

            str =~ /(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)(\w\w)/

            $1 + ':' + $2 + ':' + $3 + ':' + $4 + ':' + $5 + ':' + $6

        end # of hex_format.
 
end # of Ethernet class.

